@@ -1,5 +1,5 @@
 # Copyright 2015 Knowledge Economy Developments Ltd
-# 
+#
 # Henry Gomersall
 # heng@kedevelopments.co.uk
 #
@@ -33,20 +33,22 @@
 #
 
 from pyfftw import builders, empty_aligned, byte_align, FFTW
+from pyfftw import _supported_nptypes_complex, _supported_nptypes_real
 from pyfftw.builders import _utils as utils
-from .test_pyfftw_base import run_test_suites
+from .test_pyfftw_base import run_test_suites, require
 from ._get_default_args import get_default_args
 
 import unittest
 import numpy
 import numpy as np
-from numpy import fft as np_fft
+# import the numpy fft routines having the rfft normalization bug fix
+from .test_pyfftw_numpy_interface import np_fft, _numpy_fft_has_norm_kwarg
 import copy
 import warnings
 warnings.filterwarnings('always')
 
-complex_dtypes = (numpy.complex64, numpy.complex128, numpy.clongdouble)
-real_dtypes = (numpy.float32, numpy.float64, numpy.longdouble)
+complex_dtypes = _supported_nptypes_complex
+real_dtypes = _supported_nptypes_real
 
 def make_complex_data(shape, dtype):
     ar, ai = numpy.random.randn(2, *shape).astype(dtype)
@@ -68,7 +70,7 @@ output_dtypes = {
 
 functions = {
     'fft': 'complex',
-    'ifft': 'complex', 
+    'ifft': 'complex',
     'rfft': 'r2c',
     'irfft': 'c2r',
     'rfftn': 'r2c',
@@ -91,6 +93,7 @@ class BuildersTestFFT(unittest.TestCase):
             ((128, 32), {'axis': -1}),
             ((59, 100), {}),
             ((32, 32, 4), {'axis': 1}),
+            ((32, 32, 4), {'axis': 1, 'norm': 'ortho'}),
             ((64, 128, 16), {}),
             )
 
@@ -103,6 +106,7 @@ class BuildersTestFFT(unittest.TestCase):
             ((100,), (100, -20), IndexError, ''))
 
     realinv = False
+    has_norm_kwarg = _numpy_fft_has_norm_kwarg()
 
     def __init__(self, *args, **kwargs):
 
@@ -117,6 +121,9 @@ class BuildersTestFFT(unittest.TestCase):
             axes = self.axes_from_kwargs(kwargs)
             s = self.s_from_kwargs(test_shape, kwargs)
 
+            if not self.has_norm_kwarg and 'norm' in kwargs:
+                kwargs.pop('norm')
+
             if self.realinv:
                 test_shape = list(test_shape)
                 test_shape[axes[-1]] = test_shape[axes[-1]]//2 + 1
@@ -124,11 +131,11 @@ class BuildersTestFFT(unittest.TestCase):
 
             yield test_shape, s, kwargs
 
-    def validate_pyfftw_object(self, array_type, test_shape, dtype, 
+    def validate_pyfftw_object(self, array_type, test_shape, dtype,
             s, kwargs):
 
         input_array = array_type(test_shape, dtype)
-        
+
         # Use char because of potential MSVC related bug.
         if input_array.dtype.char == np.dtype('clongdouble').char:
             np_input_array = numpy.complex128(input_array)
@@ -147,12 +154,11 @@ class BuildersTestFFT(unittest.TestCase):
                     input_array.copy(), s, **kwargs)
 
             # We run FFT twice to check two operations don't
-            # yield different results (which they might if 
+            # yield different results (which they might if
             # the state is buggered up).
             output_array = FFTW_object(input_array.copy())
             output_array_2 = FFTW_object(input_array.copy())
 
-
             if 'axes' in kwargs:
                 axes = {'axes': kwargs['axes']}
             elif 'axis' in kwargs:
@@ -160,6 +166,9 @@ class BuildersTestFFT(unittest.TestCase):
             else:
                 axes = {}
 
+            if self.has_norm_kwarg and 'norm' in kwargs:
+                axes['norm'] = kwargs['norm']
+
             test_out_array = getattr(np_fft, self.func)(
                     np_input_array.copy(), s, **axes)
 
@@ -169,19 +178,19 @@ class BuildersTestFFT(unittest.TestCase):
                         # Make sure a warning is raised
                         self.assertIs(
                                 w[-1].category, numpy.ComplexWarning)
-        
+
         self.assertTrue(
-                numpy.allclose(output_array, test_out_array, 
+                numpy.allclose(output_array, test_out_array,
                     rtol=1e-2, atol=1e-4))
 
         self.assertTrue(
-                numpy.allclose(output_array_2, test_out_array, 
+                numpy.allclose(output_array_2, test_out_array,
                     rtol=1e-2, atol=1e-4))
 
         return FFTW_object
 
     def axes_from_kwargs(self, kwargs):
-        
+
         default_args = get_default_args(getattr(builders, self.func))
 
         if 'axis' in kwargs:
@@ -246,12 +255,12 @@ class BuildersTestFFT(unittest.TestCase):
 
     def test_valid(self):
         dtype_tuple = input_dtypes[functions[self.func]]
-        
+
         for dtype in dtype_tuple[0]:
             for test_shape, s, kwargs in self.test_data:
                 s = None
 
-                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1], 
+                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1],
                         test_shape, dtype, s, kwargs)
 
                 self.assertTrue(type(FFTW_object) == FFTW)
@@ -266,29 +275,29 @@ class BuildersTestFFT(unittest.TestCase):
         inp_dtype_tuple = input_dtypes[functions[self.func]]
         output_dtype_tuple = output_dtypes[functions[self.func]]
 
-        for input_dtype, output_dtype in zip(inp_dtype_tuple[0], 
+        for input_dtype, output_dtype in zip(inp_dtype_tuple[0],
                                              output_dtype_tuple):
 
             for test_shape, s, kwargs in self.test_data:
                 s = None
 
-                FFTW_object = self.validate_pyfftw_object(inp_dtype_tuple[1], 
+                FFTW_object = self.validate_pyfftw_object(inp_dtype_tuple[1],
                         test_shape, input_dtype, s, kwargs)
 
                 self.assertTrue(
-                    FFTW_object.output_array.dtype.char == 
+                    FFTW_object.output_array.dtype.char ==
                     np.dtype(output_dtype).char)
 
     def test_fail_on_invalid_s_or_axes(self):
         dtype_tuple = input_dtypes[functions[self.func]]
-        
+
         for dtype in dtype_tuple[0]:
 
             for test_shape, args, exception, e_str in self.invalid_args:
                 input_array = dtype_tuple[1](test_shape, dtype)
-                
+
                 self.assertRaisesRegex(exception, e_str,
-                        getattr(builders, self.func), 
+                        getattr(builders, self.func),
                         *((input_array,) + args))
 
 
@@ -297,7 +306,7 @@ class BuildersTestFFT(unittest.TestCase):
         for dtype in dtype_tuple[0]:
             for test_shape, s, kwargs in self.test_data:
 
-                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1], 
+                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1],
                         test_shape, dtype, s, kwargs)
 
                 self.assertTrue(type(FFTW_object) == FFTW)
@@ -321,12 +330,12 @@ class BuildersTestFFT(unittest.TestCase):
                     # They implicitly overwrite the input anyway
                     _kwargs['overwrite_input'] = True
 
-                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1], 
+                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1],
                         test_shape, dtype, s, _kwargs)
 
                 self.assertTrue(
                         type(FFTW_object) == utils._FFTWWrapper)
-    
+
     def test_bigger_s(self):
         dtype_tuple = input_dtypes[functions[self.func]]
         for dtype in dtype_tuple[0]:
@@ -338,7 +347,7 @@ class BuildersTestFFT(unittest.TestCase):
                 except TypeError:
                     s += 2
 
-                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1], 
+                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1],
                         test_shape, dtype, s, kwargs)
 
                 self.assertTrue(
@@ -355,11 +364,11 @@ class BuildersTestFFT(unittest.TestCase):
                 except TypeError:
                     s -= 2
 
-                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1], 
+                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1],
                         test_shape, dtype, s, kwargs)
 
                 self.assertTrue(
-                        type(FFTW_object) == utils._FFTWWrapper)                
+                        type(FFTW_object) == utils._FFTWWrapper)
 
     def test_bigger_and_smaller_s(self):
         dtype_tuple = input_dtypes[functions[self.func]]
@@ -375,15 +384,15 @@ class BuildersTestFFT(unittest.TestCase):
                     s += i * 2
                     i *= i
 
-                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1], 
+                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1],
                         test_shape, dtype, s, kwargs)
 
                 self.assertTrue(
                         type(FFTW_object) == utils._FFTWWrapper)
-    
+
     def test_auto_contiguous_input(self):
         dtype_tuple = input_dtypes[functions[self.func]]
-        
+
         for dtype in dtype_tuple[0]:
             for test_shape, s, kwargs in self.test_data:
                 _kwargs = kwargs.copy()
@@ -400,17 +409,18 @@ class BuildersTestFFT(unittest.TestCase):
                 for each_dim in test_shape:
                     _test_shape.append(each_dim*2)
                     slices.append(slice(None, None, 2))
+                slices = tuple(slices)
 
                 input_array = dtype_tuple[1](_test_shape, dtype)[slices]
                 # check the input is non contiguous
-                self.assertFalse(input_array.flags['C_CONTIGUOUS'] or 
+                self.assertFalse(input_array.flags['C_CONTIGUOUS'] or
                     input_array.flags['F_CONTIGUOUS'])
 
 
                 # Firstly check the non-contiguous case (for both
                 # FFTW and _FFTWWrapper)
                 _kwargs['auto_contiguous'] = False
-                
+
                 # We also need to make sure we're not copying due
                 # to a trivial misalignment
                 _kwargs['auto_align_input'] = False
@@ -421,7 +431,7 @@ class BuildersTestFFT(unittest.TestCase):
                 internal_input_array = FFTW_object.input_array
                 flags = internal_input_array.flags
                 self.assertTrue(input_array is internal_input_array)
-                self.assertFalse(flags['C_CONTIGUOUS'] or 
+                self.assertFalse(flags['C_CONTIGUOUS'] or
                     flags['F_CONTIGUOUS'])
 
                 FFTW_object = getattr(builders, self.func)(
@@ -440,9 +450,9 @@ class BuildersTestFFT(unittest.TestCase):
 
                 internal_input_array = FFTW_object.input_array
                 flags = internal_input_array.flags
-                self.assertTrue(flags['C_CONTIGUOUS'] or 
+                self.assertTrue(flags['C_CONTIGUOUS'] or
                     flags['F_CONTIGUOUS'])
-                
+
                 FFTW_object = getattr(builders, self.func)(
                         input_array, s2, **_kwargs)
 
@@ -454,7 +464,7 @@ class BuildersTestFFT(unittest.TestCase):
 
     def test_auto_align_input(self):
         dtype_tuple = input_dtypes[functions[self.func]]
-        
+
         for dtype in dtype_tuple[0]:
             for test_shape, s, kwargs in self.test_data:
                 _kwargs = kwargs.copy()
@@ -489,7 +499,7 @@ class BuildersTestFFT(unittest.TestCase):
 
                 self.assertTrue(FFTW_object.simd_aligned)
 
-                self.assertTrue('FFTW_UNALIGNED' not in FFTW_object.flags)                
+                self.assertTrue('FFTW_UNALIGNED' not in FFTW_object.flags)
                 FFTW_object = getattr(builders, self.func)(
                         input_array.copy(), s2, **_kwargs)
 
@@ -508,7 +518,7 @@ class BuildersTestFFT(unittest.TestCase):
             for test_shape, s, kwargs in self.test_data:
                 s = None
 
-                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1], 
+                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1],
                         test_shape, dtype, s, kwargs)
 
                 self.assertTrue(type(FFTW_object) == FFTW)
@@ -534,9 +544,9 @@ class BuildersTestFFT(unittest.TestCase):
                 except TypeError:
                     s += n_add
                     padding_slicer[axes[0]] = slice(s, None)
-
+                padding_slicer = tuple(padding_slicer)
                 # Get a valid object
-                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1], 
+                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1],
                         test_shape, dtype, s, kwargs)
 
                 internal_array = FFTW_object.input_array
@@ -559,7 +569,7 @@ class BuildersTestFFT(unittest.TestCase):
         '''
         dtype_tuple = input_dtypes[functions[self.func]]
         test_shape = (16,)
-        
+
         for dtype in dtype_tuple[0]:
             s = None
             if self.axes_kw == 'axis':
@@ -567,11 +577,11 @@ class BuildersTestFFT(unittest.TestCase):
             else:
                 kwargs = {'axes': (-1,)}
 
-            for each_effort in ('FFTW_ESTIMATE', 'FFTW_MEASURE', 
+            for each_effort in ('FFTW_ESTIMATE', 'FFTW_MEASURE',
                     'FFTW_PATIENT', 'FFTW_EXHAUSTIVE'):
-                
+
                 kwargs['planner_effort'] = each_effort
-                
+
                 FFTW_object = self.validate_pyfftw_object(
                         dtype_tuple[1], test_shape, dtype, s, kwargs)
 
@@ -580,7 +590,7 @@ class BuildersTestFFT(unittest.TestCase):
             kwargs['planner_effort'] = 'garbage'
 
             self.assertRaisesRegex(ValueError, 'Invalid planner effort',
-                    self.validate_pyfftw_object, 
+                    self.validate_pyfftw_object,
                     *(dtype_tuple[1], test_shape, dtype, s, kwargs))
 
     def test_threads_arg(self):
@@ -588,7 +598,7 @@ class BuildersTestFFT(unittest.TestCase):
         '''
         dtype_tuple = input_dtypes[functions[self.func]]
         test_shape = (16,)
-        
+
         for dtype in dtype_tuple[0]:
             s = None
             if self.axes_kw == 'axis':
@@ -597,16 +607,16 @@ class BuildersTestFFT(unittest.TestCase):
                 kwargs = {'axes': (-1,)}
 
             kwargs['threads'] = 2
-            
+
             # Should just work
             FFTW_object = self.validate_pyfftw_object(
                     dtype_tuple[1], test_shape, dtype, s, kwargs)
 
             kwargs['threads'] = 'bleh'
-            
+
             # Should not work
             self.assertRaises(TypeError,
-                    self.validate_pyfftw_object, 
+                    self.validate_pyfftw_object,
                     *(dtype_tuple[1], test_shape, dtype, s, kwargs))
 
 
@@ -614,13 +624,13 @@ class BuildersTestFFT(unittest.TestCase):
         '''Test the overwrite_input flag
         '''
         dtype_tuple = input_dtypes[functions[self.func]]
-        
+
         for dtype in dtype_tuple[0]:
             for test_shape, s, _kwargs in self.test_data:
                 s = None
 
                 kwargs = _kwargs.copy()
-                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1], 
+                FFTW_object = self.validate_pyfftw_object(dtype_tuple[1],
                         test_shape, dtype, s, kwargs)
 
                 if self.func not in ('irfft2', 'irfftn'):
@@ -643,7 +653,7 @@ class BuildersTestFFT(unittest.TestCase):
             for test_shape, s, kwargs in self.test_data:
 
                 input_array = dtype_tuple[1](test_shape, dtype)
-                
+
                 FFTW_object = getattr(
                         builders, self.func)(input_array, s, **kwargs)
 
@@ -656,7 +666,7 @@ class BuildersTestFFT(unittest.TestCase):
         '''Test the avoid_copy flag
         '''
         dtype_tuple = input_dtypes[functions[self.func]]
-        
+
         for dtype in dtype_tuple[0]:
             for test_shape, s, kwargs in self.test_data:
                 _kwargs = kwargs.copy()
@@ -672,20 +682,20 @@ class BuildersTestFFT(unittest.TestCase):
 
                 input_array = dtype_tuple[1](test_shape, dtype)
 
-                self.assertRaisesRegex(ValueError, 
+                self.assertRaisesRegex(ValueError,
                         'Cannot avoid copy.*transform shape.*',
                         getattr(builders, self.func),
                         input_array, s2, **_kwargs)
 
                 non_contiguous_shape = [
                         each_dim * 2 for each_dim in test_shape]
-                non_contiguous_slices = (
+                non_contiguous_slices = tuple(
                         [slice(None, None, 2)] * len(test_shape))
 
                 misaligned_input_array = dtype_tuple[1](
                         non_contiguous_shape, dtype)[non_contiguous_slices]
 
-                self.assertRaisesRegex(ValueError, 
+                self.assertRaisesRegex(ValueError,
                         'Cannot avoid copy.*not contiguous.*',
                         getattr(builders, self.func),
                         misaligned_input_array, s, **_kwargs)
@@ -699,7 +709,7 @@ class BuildersTestFFT(unittest.TestCase):
                 misaligned_input_array = _input_array[1:].view(
                          dtype=input_array.dtype).reshape(*test_shape)
 
-                self.assertRaisesRegex(ValueError, 
+                self.assertRaisesRegex(ValueError,
                         'Cannot avoid copy.*not aligned.*',
                         getattr(builders, self.func),
                         misaligned_input_array, s, **_kwargs)
@@ -732,6 +742,7 @@ class BuildersTestFFT2(BuildersTestFFT):
             ((128, 32), {'axes': None}),
             ((128, 32, 4), {'axes': (0, 2)}),
             ((59, 100), {'axes': (-2, -1)}),
+            ((59, 100), {'axes': (-2, -1), 'norm': 'ortho'}),
             ((64, 128, 16), {'axes': (0, 2)}),
             ((4, 6, 8, 4), {'axes': (0, 3)}),
             )
@@ -761,6 +772,7 @@ class BuildersTestFFTN(BuildersTestFFT2):
             ((128, 32, 4), {'axes': None}),
             ((64, 128, 16), {'axes': (0, 1, 2)}),
             ((4, 6, 8, 4), {'axes': (0, 3, 1)}),
+            ((4, 6, 8, 4), {'axes': (0, 3, 1), 'norm': 'ortho'}),
             ((4, 6, 8, 4), {'axes': (0, 3, 1, 2)}),
             )
 
@@ -789,8 +801,10 @@ class BuildersTestFFTWWrapper(unittest.TestCase):
 
     def setUp(self):
 
-        self.input_array_slicer = [slice(None), slice(256)]
-        self.FFTW_array_slicer = [slice(128), slice(None)]
+        require(self, '64')
+
+        self.input_array_slicer = tuple([slice(None), slice(256)])
+        self.FFTW_array_slicer = tuple([slice(128), slice(None)])
 
         self.input_array = empty_aligned((128, 512), dtype='complex128')
         self.output_array = empty_aligned((256, 256), dtype='complex128')
@@ -1092,9 +1106,9 @@ class BuildersTestUtilities(unittest.TestCase):
                 )
 
         outputs = (
-                ([slice(0, 4), slice(0, 5)], [slice(None), slice(None)]),
-                ([slice(0, 3), slice(0, 4)], [slice(None), slice(0, 4)]),
-                ([slice(0, 3), slice(0, 5)], [slice(None), slice(None)]),
+                ((slice(0, 4), slice(0, 5)), (slice(None), slice(None))),
+                ((slice(0, 3), slice(0, 4)), (slice(None), slice(0, 4))),
+                ((slice(0, 3), slice(0, 5)), (slice(None), slice(None))),
                 )
 
         for _input, _output in zip(inputs, outputs):
