@@ -247,22 +247,27 @@ class InterfacesDaskFFTTestFFT(unittest.TestCase):
                     input_array, s, **kwargs)
 
             if (functions[self.func] == 'r2c'):
-                if numpy.iscomplexobj(input_array):
+                if input_array.dtype.kind == 'c':
                     if len(w) > 0:
                         # Make sure a warning is raised
                         self.assertIs(
                                 w[-1].category, numpy.ComplexWarning)
 
+        # convert dask arrays to NumPy ones prior to calling allclose
+        output_np = output_array.compute()
+        input_np = input_array.compute()
+        test_out_array = test_out_array.compute()
+
         self.assertTrue(
-                numpy.allclose(output_array, test_out_array,
+                numpy.allclose(output_np, test_out_array,
                     rtol=1e-2, atol=1e-4))
 
-        if numpy.asanyarray(input_array).real.dtype == numpy.float16:
+        if input_np.real.dtype == numpy.float16:
             # FFTW output will never be single precision for half precision
             # inputs as there is no half-precision FFTW routine
             input_precision_dtype = numpy.float32
         else:
-            input_precision_dtype = numpy.asanyarray(input_array).real.dtype
+            input_precision_dtype = input_np.real.dtype
 
         self.assertEqual(input_precision_dtype,
                 output_array.real.dtype)
@@ -455,7 +460,7 @@ class InterfacesDaskFFTTestFFT(unittest.TestCase):
                         test_shape, dtype, s, kwargs)
 
 
-    def test_dtype_coercian(self):
+    def test_dtype_coercion(self):
         # Make sure we input a dtype that needs to be coerced
         if functions[self.func] == 'r2c':
             dtype_tuple = self.io_dtypes['complex']
